import { system, world, BlockPermutation, Direction } from '@minecraft/server';
import { decrementStack, getOppositeDirection, DirectionType } from './utils/helper';
import { directionToVector3 } from './utils/math';

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  initEvent.blockComponentRegistry.registerCustomComponent('pa_green_slab:trigger', {
  beforeOnPlayerPlace: e => { let playerEquipment = e.player.getComponent('equippable').getEquipment('Mainhand');
if (playerEquipment === undefined)
    return;
let face = getOppositeDirection(e.face);
let blockToCheck = e.block.offset(directionToVector3(face));
const blockStateDouble = 'pa:is_double';
const blockStateHalf = 'minecraft:vertical_half';
if (blockToCheck.permutation.getState(blockStateDouble))
    return;
if (blockToCheck.typeId !== playerEquipment.typeId)
    return;
if (face === 'Up' || face === 'Down') {
    const state = blockToCheck.permutation.getState(blockStateHalf);
    if ((face === 'Up' && state === 'top') ||
        (face === 'Down' && state === 'bottom')) {
        blockToCheck.dimension.playSound("use.stone", blockToCheck.location, { pitch: 0.8, volume: 1.0 });
        blockToCheck.setPermutation(BlockPermutation.resolve(blockToCheck.typeId, {
            [blockStateDouble]: true
        }));
        e.cancel = true;
        decrementStack(player);
    }
}
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('pa_green_fence_gate:trigger', {
  onPlayerInteract: e => { let face = getOppositeDirection(e.face);

const isOpen = e.block.permutation.getState("pa:open");
const sound = isOpen ? "close.wooden_trapdoor" : "open.wooden_trapdoor";

e.block.setPermutation(e.block.permutation.withState("minecraft:cardinal_direction", face.toLowerCase()));
e.block.setPermutation(e.block.permutation.withState("pa:open", !isOpen));

e.dimension.playSound(sound, e.block.center(), {
    pitch: 0.9,
    volume: 0.9,
});
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('pa_blue_brick_stair:trigger', {
  beforeOnPlayerPlace: e => { let face = e.permutationToPlace.getState('minecraft:cardinal_direction');
let half = e.permutationToPlace.getState('minecraft:vertical_half');
let blockPermutation = e.permutationToPlace;
let blockToCheck;
switch (face) {
    case 'north':
        {
            for (const direction of DirectionType.HORIZONTAL) {
                blockToCheck = e.block.offset(directionToVector3(direction));
                if (blockToCheck.typeId !== blockPermutation.type.id)
                    continue;
                const blockToCheckHalf = blockToCheck.permutation.getState('minecraft:vertical_half');
                if (blockToCheckHalf !== half)
                    continue;
                const blockToCheckDirection = blockToCheck.permutation.getState('minecraft:cardinal_direction');
                switch (direction) {
                    case Direction.North:
                        {
                            if (blockToCheckDirection === 'west') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_left');
                            }
                            if (blockToCheckDirection === 'east') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_right');
                            }
                        }
                        break;
                    case Direction.South:
                        {
                            if (blockToCheckDirection === 'east') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_left');
                            }
                            if (blockToCheckDirection === 'west') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_right');
                            }
                        }
                        break;
                    case Direction.East:
                        {
                            if (blockToCheckDirection === 'east') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_right'));
                            }
                            if (blockToCheckDirection === 'west') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_right'));
                            }
                        }
                        break;
                    case Direction.West:
                        {
                            if (blockToCheckDirection === 'west') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_left'));
                            }
                            if (blockToCheckDirection === 'east') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_left'));
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        break;
    case 'south':
        {
            blockPermutation = blockPermutation.withState('pa:south', true);
            for (const direction of DirectionType.HORIZONTAL) {
                blockToCheck = e.block.offset(directionToVector3(direction));
                if (blockToCheck.typeId !== blockPermutation.type.id)
                    continue;
                const blockToCheckHalf = blockToCheck.permutation.getState('minecraft:vertical_half');
                if (blockToCheckHalf !== half)
                    continue;
                const blockToCheckDirection = blockToCheck.permutation.getState('minecraft:cardinal_direction');
                switch (direction) {
                    case Direction.North:
                        {
                            if (blockToCheckDirection === 'west') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_left');
                            }
                            if (blockToCheckDirection === 'east') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_right');
                            }
                        }
                        break;
                    case Direction.South:
                        {
                            if (blockToCheckDirection === 'east') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_left');
                            }
                            if (blockToCheckDirection === 'west') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_right');
                            }
                        }
                        break;
                    case Direction.East:
                        {
                            if (blockToCheckDirection === 'east') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_left'));
                            }
                            if (blockToCheckDirection === 'west') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_left'));
                            }
                        }
                        break;
                    case Direction.West:
                        {
                            if (blockToCheckDirection === 'west') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_right'));
                            }
                            if (blockToCheckDirection === 'east') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_right'));
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        break;
    case 'east':
        {
            for (const direction of DirectionType.HORIZONTAL) {
                blockToCheck = e.block.offset(directionToVector3(direction));
                if (blockToCheck.typeId !== blockPermutation.type.id)
                    continue;
                const blockToCheckHalf = blockToCheck.permutation.getState('minecraft:vertical_half');
                if (blockToCheckHalf !== half)
                    continue;
                const blockToCheckDirection = blockToCheck.permutation.getState('minecraft:cardinal_direction');
                switch (direction) {
                    case Direction.East:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_left');
                            }
                            if (blockToCheckDirection === 'south') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_right');
                            }
                        }
                        break;
                    case Direction.West:
                        {
                            if (blockToCheckDirection === 'south') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_left');
                            }
                            if (blockToCheckDirection === 'north') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_right');
                            }
                        }
                        break;
                    case Direction.North:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_left'));
                            }
                            if (blockToCheckDirection === 'south') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_left'));
                            }
                        }
                        break;
                    case Direction.South:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_right'));
                            }
                            if (blockToCheckDirection === 'south') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_right'));
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        break;
    case 'west':
        {
            for (const direction of DirectionType.HORIZONTAL) {
                blockToCheck = e.block.offset(directionToVector3(direction));
                if (blockToCheck.typeId !== blockPermutation.type.id)
                    continue;
                const blockToCheckHalf = blockToCheck.permutation.getState('minecraft:vertical_half');
                if (blockToCheckHalf !== half)
                    continue;
                const blockToCheckDirection = blockToCheck.permutation.getState('minecraft:cardinal_direction');
                switch (direction) {
                    case Direction.East:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_left');
                            }
                            if (blockToCheckDirection === 'south') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_right');
                            }
                        }
                        break;
                    case Direction.West:
                        {
                            if (blockToCheckDirection === 'south') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_left');
                            }
                            if (blockToCheckDirection === 'north') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_right');
                            }
                        }
                        break;
                    case Direction.North:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_right'));
                            }
                            if (blockToCheckDirection === 'south') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_right'));
                            }
                        }
                        break;
                    case Direction.South:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_left'));
                            }
                            if (blockToCheckDirection === 'south') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_left'));
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        break;
    default:
        break;
}
e.permutationToPlace = blockPermutation;
e.block.dimension.playSound("use.stone", e.block.location, { pitch: 0.8, volume: 1.0 });
 },
});

});
